//
//  InterfaceController.swift
//  Hermez WatchKit Extension
//
//  Created by Administrator on 15/03/2016.
//  Copyright © 2016 cycling asia. All rights reserved.
//

import WatchKit
import Foundation


struct Globals {
    static var IC:InterfaceController!
}

class InterfaceController: WKInterfaceController {

    @IBOutlet weak var hands: WKInterfaceGroup!
    @IBOutlet weak var fadeIn: WKInterfaceGroup!
    
    var timer:NSTimer!
    let width = 500.0
    let height = 500.0

    override init()
    {
        super.init()
        Globals.IC = self;
    }
    
    override func awakeWithContext(context: AnyObject?) {
        super.awakeWithContext(context)
        self.beginFadeIn()
        self.drawHands()
    }

    override func willActivate() {
        self.timer?.invalidate()
        self.timer=NSTimer.scheduledTimerWithTimeInterval(1.0, target: self, selector: "drawHands", userInfo: nil, repeats: true)
    }
    
    override func didDeactivate() {
        // This method is called when watch view controller is no longer visible
        super.didDeactivate()
        self.timer?.invalidate()
    }

    func beginFadeIn()
    {
        self.fadeIn.setAlpha(1.0)
        self.animateWithDuration(0.75, animations: { self.fadeIn.setAlpha(0.0) })
    }
    
    func hideHands() {
        self.timer?.invalidate()
        fadeIn.setAlpha(1.0)
    }
    
    func getCGColor(r:Float, g:Float, b:Float, a:Float)->CGColorRef {
        return UIColor(
            red: CGFloat(r),
            green: CGFloat(g),
            blue: CGFloat(b),
            alpha: CGFloat(a)
        ).CGColor
    }

    func rotateContext(context:CGContext?, x:CGFloat, y:CGFloat, angle:CGFloat) {
        CGContextTranslateCTM(context, x, y)
        CGContextRotateCTM(context, angle)
        CGContextTranslateCTM(context, -x, -y)
    }
    
    func drawHourHand(context:CGContext?, x1:CGFloat, y1:CGFloat, angle:CGFloat) {
        let y2 = y1 - CGFloat(self.height * 0.3)
        CGContextSetShouldAntialias(context, true)
        CGContextSetStrokeColorWithColor(context, getCGColor(1.0, g: 1.0, b: 1.0, a: 1.0))
        CGContextSetRGBFillColor(context, 1.0, 1.0, 1.0, 1.0)
        CGContextSetLineWidth(context, 2.0)
        self.rotateContext(context, x:x1, y:y1, angle:angle)
        CGContextBeginPath(context)
        CGContextAddArc(context, x1, y2, 6, CGFloat(2*M_PI), CGFloat(M_PI), 1)
        CGContextMoveToPoint(context, x1 - 6, y1 - 40)
        CGContextAddLineToPoint(context, x1 - 6, y2)
        CGContextMoveToPoint(context, x1 + 6, y1 - 40)
        CGContextAddLineToPoint(context, x1 + 6, y2)
        CGContextSetLineWidth(context, 6.0)
        CGContextMoveToPoint(context, x1 - 6, y1 - 40)
        CGContextAddArc(context, x1, y1 - 40, 6, CGFloat(M_PI), CGFloat(2*M_PI), 1)
        CGContextMoveToPoint(context, x1, y1 - 35)
        CGContextAddLineToPoint(context, x1, y1)
        CGContextSetLineWidth(context, 4.0)
        CGContextAddArc(context, x1, y1, 6, 0, CGFloat(2*M_PI), 1)
        CGContextStrokePath(context)
        self.rotateContext(context, x:x1, y:y1, angle:-angle)
    }

    func drawMinuteHand(context:CGContext?, x1:CGFloat, y1:CGFloat, angle:CGFloat) {
        let y2 = y1 - CGFloat(self.width * 0.45)
        CGContextSetStrokeColorWithColor(context, getCGColor(1.0, g: 1.0, b: 1.0, a: 1.0))
        CGContextSetLineWidth(context, 2.0)
        self.rotateContext(context, x:x1, y:y1, angle:angle)
        CGContextBeginPath(context)
        CGContextAddArc(context, x1, y2, 6, CGFloat(2*M_PI), CGFloat(M_PI), 1)
        CGContextMoveToPoint(context, x1 - 6, y1 - 40)
        CGContextAddLineToPoint(context, x1 - 6, y2)
        CGContextMoveToPoint(context, x1 + 6, y1 - 40)
        CGContextAddLineToPoint(context, x1 + 6, y2)
        CGContextSetLineWidth(context, 6.0)
        CGContextMoveToPoint(context, x1 - 6, y1 - 40)
        CGContextAddArc(context, x1, y1 - 40, 6, CGFloat(M_PI), CGFloat(2*M_PI), 1)
        CGContextMoveToPoint(context, x1, y1 - 35)
        CGContextAddLineToPoint(context, x1, y1)
        CGContextSetLineWidth(context, 4.0)
        //CGContextAddArc(context, x1, y1, 6, 0, CGFloat(2*M_PI), 1)
        CGContextStrokePath(context)
        self.rotateContext(context, x:x1, y:y1, angle:-angle)
    }
    
    func drawSecondHand(context:CGContext?, x1:CGFloat, y1:CGFloat, angle:CGFloat) {
        CGContextSetStrokeColorWithColor(context, getCGColor(1.0, g: 0.50, b: 0.0, a: 1.0))
        CGContextSetLineWidth(context, 4.0)
        self.rotateContext(context, x:x1, y:y1, angle:angle)
        CGContextBeginPath(context)
        CGContextAddArc(context, x1, y1, 4, 0, CGFloat(2*M_PI), 1)
        CGContextMoveToPoint(context, x1, y1 + 40)
        CGContextAddLineToPoint(context, x1, y1 - CGFloat(self.height * 0.475))
        CGContextMoveToPoint(context, x1, y1)
        CGContextAddArc(context, x1, y1, 4, 0, CGFloat(2*M_PI), 1)
        CGContextStrokePath(context)
        CGContextSetLineWidth(context, 2.0)
        CGContextSetStrokeColorWithColor(context, getCGColor(0.0, g: 0.0, b: 0.0, a: 1.0))
        CGContextAddArc(context, x1, y1, 2, 0, CGFloat(2*M_PI), 1)
        CGContextStrokePath(context)
        self.rotateContext(context, x:x1, y:y1, angle:-angle)
    }

    func drawHands() {
        var t = time_t()
        time(&t)
        let x = localtime(&t) // returns UnsafeMutablePointer

        let sec = x.memory.tm_sec
        let min = x.memory.tm_min
        var hour = x.memory.tm_hour;
        
        if hour>12 {
            hour-=12
        }
    
        let mAng=deg2rad(CGFloat(min*6))
        let hAng=deg2rad(CGFloat(hour*30))+mAng/12
        let sAng=deg2rad(CGFloat(sec*6))

        UIGraphicsBeginImageContext(CGSize(width: self.width, height: self.height))
        let context = UIGraphicsGetCurrentContext()
        
        self.drawHourHand(context, x1:CGFloat(self.width * 0.5), y1:CGFloat(self.height * 0.5), angle:hAng)
        self.drawMinuteHand(context, x1:CGFloat(self.width * 0.5), y1:CGFloat(self.height * 0.5), angle:mAng)
        self.drawSecondHand(context, x1:CGFloat(self.width * 0.5), y1:CGFloat(self.height * 0.5), angle:sAng)
        
        let uimg = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        hands.setBackgroundImage(uimg)
        hands.setHidden(false)
        
        // update the app title with the current date
        let df=NSDateFormatter()
        df.dateFormat="[dd]"
        self.setTitle(df.stringFromDate(NSDate()))
    }
    
    private func deg2rad(deg:CGFloat)->CGFloat{
        return CGFloat(Double(deg)*M_PI/Double(180))
    }
}
